@@ -3,6 +3,7 @@
 # Copyright (c) 2013 Paul Tagliamonte <paultag@debian.org>
 # Copyright (c) 2013 Julien Danjou <julien@danjou.info>
 # Copyright (c) 2013 Nicolas Dandrimont <nicolas.dandrimont@crans.org>
+# Copyright (c) 2013 James King <james@agentultra.com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -35,8 +36,8 @@ from hy.models.float import HyFloat
 from hy.models.list import HyList
 from hy.models.dict import HyDict
 
-from hy.macros import require, process
-from hy.util import str_type
+from hy.macros import require, macroexpand
+from hy._compat import str_type
 import hy.importer
 
 import traceback
@@ -154,7 +155,8 @@ class Result(object):
     The Result object is interoperable with python AST objects: when an AST
     object gets added to a Result object, it gets converted on-the-fly.
     """
-    __slots__ = ("imports", "stmts", "temp_variables", "_expr", "__used_expr")
+    __slots__ = ("imports", "stmts", "temp_variables",
+                 "_expr", "__used_expr", "contains_yield")
 
     def __init__(self, *args, **kwargs):
         if args:
@@ -165,12 +167,14 @@ class Result(object):
         self.stmts = []
         self.temp_variables = []
         self._expr = None
+        self.contains_yield = False
 
         self.__used_expr = False
 
         # XXX: Make sure we only have AST where we should.
         for kwarg in kwargs:
-            if kwarg not in ["imports", "stmts", "expr", "temp_variables"]:
+            if kwarg not in ["imports", "contains_yield", "stmts", "expr",
+                             "temp_variables"]:
                 raise TypeError(
                     "%s() got an unexpected keyword argument '%s'" % (
                         self.__class__.__name__, kwarg))
@@ -282,13 +286,21 @@ class Result(object):
         result.stmts = self.stmts + other.stmts
         result.expr = other.expr
         result.temp_variables = other.temp_variables
+        result.contains_yield = False
+        if self.contains_yield or other.contains_yield:
+            result.contains_yield = True
+
         return result
 
     def __str__(self):
-        return "Result(imports=[%s], stmts=[%s], expr=%s)" % (
+        return (
+            "Result(imports=[%s], stmts=[%s], "
+            "expr=%s, contains_yield=%s)"
+        ) % (
             ", ".join(ast.dump(x) for x in self.imports),
             ", ".join(ast.dump(x) for x in self.stmts),
             ast.dump(self.expr) if self.expr else None,
+            self.contains_yield
         )
 
 
@@ -318,7 +330,7 @@ def _raise_wrong_args_number(expression, error):
                                len(expression)))
 
 
-def checkargs(exact=None, min=None, max=None):
+def checkargs(exact=None, min=None, max=None, even=None):
     def _dec(fn):
         def checker(self, expression):
             if exact is not None and (len(expression) - 1) != exact:
@@ -335,6 +347,14 @@ def checkargs(exact=None, min=None, max=None):
                     expression,
                     "`%%s' needs at most %d arguments, got %%d" % (max))
 
+            is_even = not((len(expression) - 1) % 2)
+            if even is not None and is_even != even:
+                even_str = "even" if even else "odd"
+                _raise_wrong_args_number(
+                    expression,
+                    "`%%s' needs an %s number of arguments, got %%d"
+                    % (even_str))
+
             return fn(self, expression)
 
         return checker
@@ -399,7 +419,7 @@ class HyASTCompiler(object):
 
     def compile(self, tree):
         try:
-            tree = process(tree, self.module_name)
+            tree = macroexpand(tree, self.module_name)
             _type = type(tree)
             ret = self.compile_atom(_type, tree)
             if ret:
@@ -590,7 +610,7 @@ class HyASTCompiler(object):
                                                                          level)
                 imports.update(f_imports)
                 if splice:
-                    to_add = f_contents
+                    to_add = HyExpression([HySymbol("list"), f_contents])
                 else:
                     to_add = HyList([f_contents])
 
@@ -599,7 +619,7 @@ class HyASTCompiler(object):
             return imports, HyExpression([HySymbol(name),
                                           contents]).replace(form), False
 
-        elif isinstance(form, HySymbol):
+        elif isinstance(form, (HySymbol, HyLambdaListKeyword)):
             return imports, HyExpression([HySymbol(name),
                                           HyString(form)]).replace(form), False
 
@@ -946,31 +966,6 @@ class HyASTCompiler(object):
                              col_offset=expression.start_column)
         return ret
 
-    @builds("print")
-    def compile_print_expression(self, expr):
-        call = expr.pop(0)  # print
-        values, ret = self._compile_collect(expr)
-
-        if sys.version_info[0] >= 3:
-            call = self.compile(call)
-            ret += call
-            ret += ast.Call(func=call.expr,
-                            args=values,
-                            keywords=[],
-                            starargs=None,
-                            kwargs=None,
-                            lineno=expr.start_line,
-                            col_offset=expr.start_column)
-        else:
-            ret += ast.Print(
-                lineno=expr.start_line,
-                col_offset=expr.start_column,
-                dest=None,
-                values=values,
-                nl=True)
-
-        return ret
-
     @builds("break")
     def compile_break_expression(self, expr):
         ret = ast.Break(lineno=expr.start_line,
@@ -1011,7 +1006,7 @@ class HyASTCompiler(object):
     @checkargs(max=1)
     def compile_yield_expression(self, expr):
         expr.pop(0)
-        ret = Result()
+        ret = Result(contains_yield=True)
 
         value = None
         if expr != []:
@@ -1130,25 +1125,28 @@ class HyASTCompiler(object):
             ctx=ast.Load())
 
     @builds("assoc")
-    @checkargs(3)
+    @checkargs(min=3, even=False)
     def compile_assoc_expression(self, expr):
         expr.pop(0)  # assoc
         # (assoc foo bar baz)  => foo[bar] = baz
         target = self.compile(expr.pop(0))
-        key = self.compile(expr.pop(0))
-        val = self.compile(expr.pop(0))
+        ret = target
+        i = iter(expr)
+        for (key, val) in ((self.compile(x), self.compile(y))
+                           for (x, y) in zip(i, i)):
 
-        return target + key + val + ast.Assign(
-            lineno=expr.start_line,
-            col_offset=expr.start_column,
-            targets=[
-                ast.Subscript(
-                    lineno=expr.start_line,
-                    col_offset=expr.start_column,
-                    value=target.force_expr,
-                    slice=ast.Index(value=key.force_expr),
-                    ctx=ast.Store())],
-            value=val.force_expr)
+            ret += key + val + ast.Assign(
+                lineno=expr.start_line,
+                col_offset=expr.start_column,
+                targets=[
+                    ast.Subscript(
+                        lineno=expr.start_line,
+                        col_offset=expr.start_column,
+                        value=target.force_expr,
+                        slice=ast.Index(value=key.force_expr),
+                        ctx=ast.Store())],
+                value=val.force_expr)
+        return ret
 
     @builds("with_decorator")
     @checkargs(min=1)
@@ -1178,11 +1176,18 @@ class HyASTCompiler(object):
             thing = self._storeize(self.compile(args.pop(0)))
 
         body = self._compile_branch(expr)
-        body += body.expr_as_stmt()
 
-        if not body.stmts:
-            body += ast.Pass(lineno=expr.start_line,
-                             col_offset=expr.start_column)
+        var = self.get_anon_var()
+        name = ast.Name(id=ast_str(var), arg=ast_str(var),
+                        ctx=ast.Store(),
+                        lineno=expr.start_line,
+                        col_offset=expr.start_column)
+
+        # Store the result of the body in a tempvar
+        body += ast.Assign(targets=[name],
+                           value=body.force_expr,
+                           lineno=expr.start_line,
+                           col_offset=expr.start_column)
 
         the_with = ast.With(context_expr=ctx.force_expr,
                             lineno=expr.start_line,
@@ -1194,7 +1199,16 @@ class HyASTCompiler(object):
             the_with.items = [ast.withitem(context_expr=ctx.force_expr,
                                            optional_vars=thing)]
 
-        return ctx + the_with
+        ret = ctx + the_with
+        # And make our expression context our temp variable
+        expr_name = ast.Name(id=ast_str(var), arg=ast_str(var),
+                             ctx=ast.Load(),
+                             lineno=expr.start_line,
+                             col_offset=expr.start_column)
+
+        ret += Result(expr=expr_name, temp_variables=[expr_name, name])
+
+        return ret
 
     @builds(",")
     def compile_tuple(self, expr):
@@ -1445,9 +1459,6 @@ class HyASTCompiler(object):
             if ret:
                 return ret
 
-            if fn in _stdlib:
-                self.imports[_stdlib[fn]].add(fn)
-
             if fn.startswith("."):
                 # (.split "test test") -> "test test".split()
 
@@ -1481,7 +1492,6 @@ class HyASTCompiler(object):
         return func + ret
 
     @builds("def")
-    @builds("setf")
     @builds("setv")
     @checkargs(2)
     def compile_def_expression(self, expression):
@@ -1541,6 +1551,8 @@ class HyASTCompiler(object):
                        body=body.stmts,
                        orelse=orel.stmts)
 
+        ret.contains_yield = body.contains_yield
+
         return ret
 
     @builds("while")
@@ -1558,6 +1570,8 @@ class HyASTCompiler(object):
                          lineno=expr.start_line,
                          col_offset=expr.start_column)
 
+        ret.contains_yield = body.contains_yield
+
         return ret
 
     @builds(HyList)
@@ -1601,9 +1615,12 @@ class HyASTCompiler(object):
             return ret
 
         if body.expr:
-            body += ast.Return(value=body.expr,
-                               lineno=body.expr.lineno,
-                               col_offset=body.expr.col_offset)
+            if body.contains_yield:
+                body += body.expr_as_stmt()
+            else:
+                body += ast.Return(value=body.expr,
+                                   lineno=body.expr.lineno,
+                                   col_offset=body.expr.col_offset)
 
         if not body.stmts:
             body += ast.Pass(lineno=expression.start_line,
@@ -1647,6 +1664,17 @@ class HyASTCompiler(object):
 
         body = Result()
 
+        # grab the doc string, if there is one
+        if expression and isinstance(expression[0], HyString):
+            docstring = expression.pop(0)
+            symb = HySymbol("__doc__")
+            symb.start_line = docstring.start_line
+            symb.start_column = docstring.start_column
+            body += self._compile_assign(symb, docstring,
+                                         docstring.start_line,
+                                         docstring.start_column)
+            body += body.expr_as_stmt()
+
         if expression:
             try:
                 body_expression = iter(expression.pop(0))
@@ -1757,6 +1785,9 @@ class HyASTCompiler(object):
             )
             return ret
 
+        if symbol in _stdlib:
+            self.imports[_stdlib[symbol]].add(symbol)
+
         return ast.Name(id=ast_str(symbol),
                         arg=ast_str(symbol),
                         ctx=ast.Load(),
