@@ -18,16 +18,21 @@
 # FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 # DEALINGS IN THE SOFTWARE.
 
-from hy.lex.machine import Machine
-from hy.lex.states import Idle, LexException
+from rply.errors import LexingError
+
+from hy.lex.exceptions import LexException, PrematureEndOfInput  # NOQA
+from hy.lex.lexer import lexer
+from hy.lex.parser import parser
 
 
 def tokenize(buf):
     """
     Tokenize a Lisp file or string buffer into internal Hy objects.
     """
-    machine = Machine(Idle, 1, 0)
-    machine.process(buf)
-    if type(machine.state) != Idle:
-        raise LexException("Incomplete Lex.")
-    return machine.nodes
+    try:
+        return parser.parse(lexer.lex(buf))
+    except LexingError as e:
+        pos = e.getsourcepos()
+        raise LexException(
+            "Could not identify the next token at line %s, column %s" % (
+                pos.lineno, pos.colno))
